#include <iostream>
#include <cmath>
#include <random>
/*
  On utilise la bibliotheque random, presente seulement depuis C++ 11.
  Pour compiler ce programme avec g++, ajouter l'option -std=c++11.

  Sinon, remplacer l'utilisation de mt19937 par l'utilisation de rand.
*/ 


/*
  Rappel : si X est une variable aleatoire de fonction de
  repartition F et que G est l'inverse generalise de F
  (c'est-a-dire G(t) = inf {x, F(x) >= t} ), alors la variable
  G(U), ou U est uniforme sur [0,1] suit la meme loi que X.
 */


const int N = 100000; // Nombre de simulation dans les calculs par Monte Carlo
const double p = 0.3; // Parametre des lois de Bernoulli


double unif(void)
// Simulation d'une loi uniforme sur [0,1]
{
  static std::mt19937 generateur(12345);
  std::uniform_real_distribution<double> U;
  return U(generateur);
}

double bernoulli()
{
  return (unif()<p) ? 1 : 0;
  /*
	On tire U de loi uniforme sur [0,1].
	On renvoie 1 si U<p et 0 sinon.
  */
}


double normale(void)
{
  /*
	Methode de Box-Muller : si U et V sont uniformes sur [0,1], alors
	sqrt(-2log(U)) * cos(2piV) suit la loi normale centree reduite.
  */
  return sqrt(-2*log(unif())) * cos(2*M_PI*unif());  
}


double expo(void)
{
  // La methode de la fonction de repartition inverse montre que -log(U),
  // pour U uniforme sur [0,1] suit la loi exponentielle de parametre 1.
  return -log(unif());
}


double cauchy(void)
{
  // Toujours par la methode de la fonction de repartition inverse, tan(U) suit
  // la loi de Cauchy, si U est uniforme sur [-pi/2, pi/2]
  return tan( M_PI * (unif()-0.5) );
}

double densite_expl(void)
{
  // Methode de la fonction de repartition inverse : si U est
  // uniforme sur [0,1], alors U^-1/2 a pour densite 2x^-3 sur [1,+oo[.
  return 1/sqrt(unif());
}


void affiche_intervalle(double somme, double somme_carres)
/*
  Affiche un intervalle de confiance a 95% sur la moyenne de X
  a partir de :
  - la somme des x_i (somme)
  - la somme des x_i^2 (somme_carres)
  La taille de l'echantillon considere est N.
*/
{
  double moyenne, variance;
  moyenne = somme/N;
  variance = somme_carres/N - moyenne*moyenne;

  double largeur = 1.96*sqrt(variance/N);

  std::cout << "Intervalle de confiance a 95% : [ " << moyenne-largeur << " , " << moyenne+largeur << " ], centre en " << moyenne << std::endl;
}



void calcul_IC(double (*f)(void))
{
  double somme, somme_carres, X;
  somme = 0;
  somme_carres = 0;
  for (int i=0; i<N; i++)
	{
	  X = f();
	  somme += X;
	  somme_carres += X*X;
	}
  affiche_intervalle(somme, somme_carres);
}



int main(void)
{
  
  std::cout << "Moyenne d'une loi uniforme sur [0,1] :\n";
  calcul_IC(unif);
  std::cout << "Moyenne theorique : 0.5\n\n";
  
  std::cout << "Moyenne d'une loi de Bernoulli de parametre 0.3 :\n";
  calcul_IC(bernoulli);
  std::cout << "Moyenne theorique : 0.3\n\n";
  
  std::cout << "Moyenne d'une loi N(0,1) :\n";
  calcul_IC(normale);
  std::cout << "Moyenne theorique : 0\n\n";
  
  std::cout << "Moyenne d'une loi exponentielle de parametre 1 :\n";
  calcul_IC(expo);
  std::cout << "Moyenne theorique : 1\n\n";
  
  std::cout << "Moyenne d'une loi de Cauchy :\n";
  calcul_IC(cauchy);
  std::cout << "Pas de moyenne : la loi de Cauchy n'est pas integrable\n\n";
  
  std::cout << "Moyenne d'une loi de densite 2x^-3 sur [1,+oo[ :\n";
  calcul_IC(densite_expl);
  std::cout << "Moyenne theorique : 2. Noter que cette loi n'est pas de carre integrable, et que l'intervalle de confiance n'a donc pas de justification theorique.\n\n";
  
  return 0;
}

